'Der Datentyp "FieldType" wird fr das eigentliche Spielfeld
'benutzt. Normal ist es nmlich wenig sinnvoll, ein Spielfeld
'aus vielen einzelnen Objekten aufzubauen, die beliebige
'Position haben knnen. Diese Variante ist wesentlich schneller,
'hat aber den Nachteil, da die Felder auf dem Spielfeld immer
'gleich gro sein mssen. Jedoch knnen spezielle Teile des
'Spielfelds, wie z.B. Teleporter, als Objekte realisiert werden.

Type FieldType
    V As Integer    'Visible - Ist das Feld berhaupt belegt?
    S As Sprite     'Sprite, der fr das Feld benutzt wird
    B As Integer    'Block - handelt es sich um eine solid Wand?
End Type


'Nun wird der Typ ItemType definiert. Er wird fr die Items
'(also fr die Gegenstnde) benutzt, die die Objekte bei sich
'haben

Type ItemType
    T As Integer    'Typ des Items
    E As Integer    '"Energie" (bzw. verbleibende Zeit fr Extras...)
End Type

'Es folgt die Definition fr ObjType. Dieser Typ wird fr alle
'"Objekte" auf dem Spielfeld benutzt.

Type ObjType
    X As Integer    'X-Position
    Y As Integer    'Y-Position
    OX As Integer   'Vorherige Position (X)
    OY As Integer   'Vorherige Position (Y)
    
    SX As Integer   'X-Geschwindigkeit
    SY As Integer   'Y-Geschwindigkeit

    E As Integer    'Energie
    T As Integer    'Typ des Objekts
    FRAME As Sprite 'Sprite des Objekts
    
    R1 As Integer   'Reserve-Wert #1
    R2 As Integer   'Reserve-Wert #2
    R3 As Integer   'Reserve-Wert #3
    
    V As Integer    'Fr andere Objekte "sichtbar"
    
    ITEM(20) As ItemType    'Welche Items hat das Objekt bei sich?
End Type

Global ObjDummy As ObjType  'Dummy-Objekt
Global MAX_DIFF


'Das Datenfeld FIELD() enthlt das Spielfeld mit den Wnden und
'Tren.

Global FIELD() As FieldType
Global FIELD_W              'FIELD_W = Breite eines Feldes
Global FIELD_H              'FIELD_H = Hhe eines Feldes
Global FIELD_X              'Breite des Spielfeldes (in Feldern)
Global FIELD_Y              'Hhe des Spielfeldes (in Feldern)


'Nun einige globale Konstanten

Global Const ID_COLL = 1
Global Const ID_COLL_WALL = 2
Global Const ID_HIT = 3
Global Const ID_DRAW = 4
Global Const ID_MOVE = 5

'
' Zeichnet das komplette (sichtbare) Spielfeld an die Position
' X, Y.
'
Sub FIELD_DRAW (DISPLAY As PictureBox, X, Y)

    For AX = 0 To FIELD_X
        For AY = 0 To FIELD_Y

            If FIELD(AX, AY).V Then
                
                FX = X + AX * FIELD_W
                FY = Y + AY * FIELD_H

                If FX <= DISPLAY.ScaleWidth And FY <= DISPLAY.ScaleHeight Then
                    If FX + FIELD_W >= 0 And FY + FIELD_H >= 0 Then
                    
                        SPRITE_DRAW_SOLID FIELD(AX, AY).S, FX, FY, DISPLAY
                        
                    End If
                End If
            End If
            
        Next AY
    Next AX

End Sub

'
' Setzt ein Feld des Spielfeldes
'
Sub FIELD_SET (X, Y, V, S As Sprite, B)

    FIELD(X, Y).V = V
    FIELD(X, Y).S = S
    FIELD(X, Y).B = B

    FIELD(X, Y).S.NACH.W = FIELD_W
    FIELD(X, Y).S.NACH.H = FIELD_H
    
End Sub

'
' Legt die Dimensionen des Spielfeldes fest
'
Sub FIELD_SIZE (X, Y, W, H)

    ReDim FIELD(X, Y)

    FIELD_X = X
    FIELD_Y = Y
    FIELD_W = W
    FIELD_H = H

    For AX = 0 To X
        For AY = 0 To Y
            FIELD(AX, AY).V = 0
        Next AY
    Next AX

End Sub

Sub OBJ_ANIMATE (C1, Max1, C2, Max2)
                    
    C1 = C1 + 1
    If C1 > Max1 Then
        C1 = 0
        C2 = C2 + 1
        If C2 > Max2 Then C2 = 0
    End If

End Sub

'
' Lscht alle Eigenschaften des bergebenen Objekts und
' deaktiviert es damit
'
Sub OBJ_CLEAR (O As ObjType)
    
    O.X = 0
    O.Y = 0
    O.E = 0
    O.T = 0
    O.V = 0
    O.SX = 0
    O.SY = 0
    
    O.R1 = 0
    O.R2 = 0
    O.R3 = 0
    
    For I = 0 To UBound(O.ITEM)
        O.ITEM(I).T = 0
    Next I

End Sub

'
' Prft, ob das Objekt mit einer Wand (o..) auf dem Spielfeld
' kollidiert. Wenn ja wird ein entsprechendes Ereignis and das
' Objekt abgegeben
'
Sub OBJ_COLL_FIELD (C As ObjType)
    
    If C.T = 0 Then Exit Sub

    If OBJ_FIELD_ISFREE(C.X, C.Y, C.FRAME.NACH.W, C.FRAME.NACH.H) = 0 Then
        OBJ_EVENT C, ID_COLL_WALL, ObjDummy, 0, 0
    End If

End Sub

'
' Prft, ob ein Objekt mit einem anderen Objekt kollidiert ist.
' Wenn ja, so wird die entsprechende "Ereignisprozedur"
' der Objekte aufgerufen
'
Sub OBJ_COLL_OBJ (C1 As ObjType, C2 As ObjType)

    If C1.T = 0 Then Exit Sub
    If C2.T = 0 Then Exit Sub

    X1 = C1.X
    Y1 = C1.Y
    W1 = C1.FRAME.NACH.W
    H1 = C1.FRAME.NACH.H
    
    X2 = C2.X
    Y2 = C2.Y
    W2 = C2.FRAME.NACH.W
    H2 = C2.FRAME.NACH.H

    If X1 + W1 - MAX_DIFF > X2 And X1 < X2 + W2 - MAX_DIFF Then
        If Y1 + H1 - MAX_DIFF > Y2 And Y1 < Y2 + H2 - MAX_DIFF Then

            OBJ_EVENT C1, ID_COLL, C2, 0, 0
            OBJ_EVENT C2, ID_COLL, C1, 0, 0

        End If
    End If

End Sub

Function OBJ_DOWN (C As ObjType, D)

    OBJ_DOWN = OBJ_FIELD_ISFREE(C.X, C.Y + D, C.FRAME.NACH.W, C.FRAME.NACH.H)

End Function

'
' Zeichnet das bergebene Objekt relativ zu der Position
' X,Y
'
Sub OBJ_DRAW (O As ObjType, X, Y, PicBox As PictureBox)

    If O.T = 0 Then Exit Sub

    OBJ_EVENT O, ID_DRAW, ObjDummy, 0, 0

    XX = X + O.X
    YY = Y + O.Y
    SPRITE_DRAW O.FRAME, XX, YY, PicBox
    
End Sub

'
' Event-Handler fr Objekte
'
Sub OBJ_EVENT (Self As ObjType, MESSAGE, OTHER As ObjType, ByVal Par1, ByVal Par2)

    USER_EVENT Self, MESSAGE, OTHER, Par1, Par2

End Sub

Function OBJ_FIELD_ISFREE (X, Y, W, H)
    
    X1 = X \ FIELD_W
    Y1 = Y \ FIELD_H
    X2 = (X + W) \ FIELD_W
    Y2 = (Y + H) \ FIELD_H
    
    For AX = X1 To X2
        For AY = Y1 To Y2

            If AX >= 0 And AY >= 0 And AX <= FIELD_X And AY <= FIELD_Y Then

                If FIELD(AX, AY).B And FIELD(AX, AY).V Then

                    FX = AX * FIELD_W
                    FY = AY * FIELD_H

                    If X + W - MAX_DIFF > FX And X < FX + FIELD_W - MAX_DIFF Then
                        If Y + H - MAX_DIFF > FY And Y < FY + FIELD_H - MAX_DIFF Then

                            OBJ_FIELD_ISFREE = 0
                            Exit Function
                    
                        End If
                    End If

                End If
            Else
                OBJ_FIELD_ISFREE = 0
                Exit Function
            End If
            
        Next AY
    Next AX
                            
    OBJ_FIELD_ISFREE = 1

End Function

'
' Nimmt dem Objekt das Item weg (falls berhaupt vorhanden)
'
Sub OBJ_ITEM_DROP (O As ObjType, ITEM)
    
    If O.T = 0 Then Exit Sub
    
    For I = 0 To UBound(O.ITEM)
        If O.ITEM(I).T = ITEM Then
            O.ITEM(I).T = 0
            Exit Sub
        End If
    Next I

End Sub

'
' Holt die Energie eines Items
'
Function OBJ_ITEM_ENERGY_GET (O As ObjType, ITEM)
    
    OBJ_ITEM_ENERGY_GET = OBJ_ITEM_HAVE(O, ITEM)

End Function

'
' ndert die Energie eines Items
'
Sub OBJ_ITEM_ENERGY_SET (O As ObjType, ITEM, Energy)
    
    If O.T = 0 Then Exit Sub

    For I = 0 To UBound(O.ITEM)
        If O.ITEM(I).T = ITEM Then
            O.ITEM(I).E = Energy
            If Energy <= 0 Then
                O.ITEM(I).T = 0
            End If
            Exit Sub
        End If
    Next I
    OBJ_ITEM_GET O, ITEM, Energy

End Sub

'
' Gibt einem Objekt das Item mit der Nummer I
'
Sub OBJ_ITEM_GET (O As ObjType, ITEM, Energy)

    If O.T = 0 Then Exit Sub

    For I = 0 To UBound(O.ITEM)
        If O.ITEM(I).T = 0 Then
            O.ITEM(I).T = ITEM
            O.ITEM(I).E = Energy
            Exit Sub
        End If
    Next I

End Sub

'
' Liefert zurck, ob das Objekt das entsprechende Item
' hat
'
Function OBJ_ITEM_HAVE (O As ObjType, ITEM)

    If O.T = 0 Then Exit Function

    For I = 0 To UBound(O.ITEM)
        If O.ITEM(I).T = ITEM Then
            OBJ_ITEM_HAVE = O.ITEM(I).E
            Exit Function
        End If
    Next I

End Function

Function OBJ_LEFT (C As ObjType, D)

    OBJ_LEFT = OBJ_FIELD_ISFREE(C.X - D, C.Y, (C.FRAME.NACH.W), (C.FRAME.NACH.H))

End Function

Function OBJ_RIGHT (C As ObjType, D)

    OBJ_RIGHT = OBJ_FIELD_ISFREE(C.X + D, C.Y, (C.FRAME.NACH.W), (C.FRAME.NACH.H))

End Function

'
' Setzt das bergebene Objekt O auf die bergebenen
' Werte
'
Sub OBJ_SET (O As ObjType, X, Y, E, SX, SY, T, FRAME As Sprite, V)

    O.X = X
    O.Y = Y
    O.OX = X
    O.OY = Y
    O.E = E
    O.T = T
    O.FRAME = FRAME
    O.V = V
    
    O.SX = SX
    O.SY = SY

End Sub

Function OBJ_UP (C As ObjType, D)

    OBJ_UP = OBJ_FIELD_ISFREE(C.X, C.Y - D, C.FRAME.NACH.W, C.FRAME.NACH.H)

End Function

