Attribute VB_Name = "MODUL_OBJECT"
'Der Datentyp "FieldType" wird fr das eigentliche Spielfeld
'benutzt. Normal ist es nmlich wenig sinnvoll, ein Spielfeld
'aus vielen einzelnen Objekten aufzubauen, die beliebige
'Position haben knnen. Diese Variante ist wesentlich schneller,
'hat aber den Nachteil, da die Felder auf dem Spielfeld immer
'gleich gro sein mssen. Jedoch knnen spezielle Teile des
'Spielfelds, wie z.B. Teleporter, als Objekte realisiert werden.
Public Type FieldType
  V As Long      'Visible - Ist das Feld berhaupt belegt?
  S As Sprite    'Sprite, der fr das Feld benutzt wird
  B As Long     'Block - handelt es sich um eine solid Wand?
End Type


'Nun wird der Typ ItemType definiert. Er wird fr die Items
'(also fr die Gegenstnde) benutzt, die die Objekte bei sich
'haben
Public Type ItemType
  T As Long    'Typ des Items
  E As Long    '"Energie" (bzw. verbleibende Zeit fr Extras...)
End Type

'Es folgt die Definition fr ObjType. Dieser Typ wird fr alle
'"Objekte" auf dem Spielfeld benutzt.
Public Type ObjType
  x As Long       'X-Position
  y As Long       'Y-Position
  OX As Long      'Vorherige Position (X)
  OY As Long      'Vorherige Position (Y)
    
  SX As Long      'X-Geschwindigkeit
  SY As Long      'Y-Geschwindigkeit

  E As Long       'Energie
  T As Long       'Typ des Objekts
  FRAME As Sprite 'Sprite des Objekts
    
  R1 As Long      'Reserve-Wert #1
  R2 As Long      'Reserve-Wert #2
  R3 As Long      'Reserve-Wert #3
    
  V As Long       'Fr andere Objekte "sichtbar"
  
  ITEM(20) As ItemType    'Welche Items hat das Objekt bei sich?
End Type

Public ObjDummy As ObjType  'Dummy-Objekt
Public MAX_DIFF As Long


'Das Datenfeld FIELD() enthlt das Spielfeld mit den Wnden und
'Tren.
Public FIELD() As FieldType
Public FIELD_W As Long 'FIELD_W = Breite eines Feldes
Public FIELD_H As Long 'FIELD_H = Hhe eines Feldes
Public FIELD_X As Long 'Breite des Spielfeldes (in Feldern)
Public FIELD_Y As Long 'Hhe des Spielfeldes (in Feldern)


'Nun einige globale Konstanten
Public Const ID_COLL = 1
Public Const ID_COLL_WALL = 2
Public Const ID_HIT = 3
Public Const ID_DRAW = 4
Public Const ID_MOVE = 5

' Zeichnet das komplette (sichtbare) Spielfeld an die Position X, Y.
Public Sub FIELD_DRAW(DISPLAY As PictureBox, x As Long, y As Long)
  Dim aX As Long, aY As Long, fX As Long, fY As Long

    For aX = 0 To FIELD_X
      For aY = 0 To FIELD_Y
        If FIELD(aX, aY).V Then
          fX = x + aX * FIELD_W
          fY = y + aY * FIELD_H

          If fX <= DISPLAY.ScaleWidth And fY <= DISPLAY.ScaleHeight Then
            If fX + FIELD_W >= 0 And fY + FIELD_H >= 0 Then
              Call SPRITE_DRAW_SOLID(FIELD(aX, aY).S, fX, fY, DISPLAY)
            End If
          End If
        End If
      Next aY
    Next aX
End Sub

' Setzt ein Feld des Spielfeldes
Public Sub FIELD_SET(x As Long, y As Long, V As Long, S As Sprite, B As Long)
  FIELD(x, y).V = V
  FIELD(x, y).S = S
  FIELD(x, y).B = B

  FIELD(x, y).S.NACH.w = FIELD_W
  FIELD(x, y).S.NACH.h = FIELD_H
End Sub

' Legt die Dimensionen des Spielfeldes fest
Public Sub FIELD_SIZE(x As Long, y As Long, w As Long, h As Long)
  Dim aX As Long, aY As Long
  
    ReDim FIELD(x, y)

    FIELD_X = x
    FIELD_Y = y
    FIELD_W = w
    FIELD_H = h

    For aX = 0 To x
        For aY = 0 To y
            FIELD(aX, aY).V = 0
        Next aY
    Next aX
End Sub

Public Sub OBJ_ANIMATE(C1 As Long, Max1 As Long, C2 As Long, Max2 As Long)
  C1 = C1 + 1
  
  If C1 > Max1 Then
    C1 = 0
    C2 = C2 + 1
    If C2 > Max2 Then C2 = 0
  End If
End Sub


' Lscht alle Eigenschaften des bergebenen Objekts und
' deaktiviert es damit
Public Sub OBJ_CLEAR(O As ObjType)
  Dim i As Long
  
    O.x = 0
    O.y = 0
    O.E = 0
    O.T = 0
    O.V = 0
    O.SX = 0
    O.SY = 0
    
    O.R1 = 0
    O.R2 = 0
    O.R3 = 0
    
    For i = 0 To UBound(O.ITEM)
      O.ITEM(i).T = 0
    Next i
End Sub

' Prft, ob das Objekt mit einer Wand (o..) auf dem Spielfeld
' kollidiert. Wenn ja wird ein entsprechendes Ereignis and das
' Objekt abgegeben
Public Sub OBJ_COLL_FIELD(C As ObjType)
  If C.T = 0 Then Exit Sub

  If OBJ_FIELD_ISFREE(C.x, C.y, C.FRAME.NACH.w, C.FRAME.NACH.h) = 0 Then
    Call OBJ_EVENT(C, ID_COLL_WALL, ObjDummy, 0, 0)
  End If
End Sub

' Prft, ob ein Objekt mit einem anderen Objekt kollidiert ist.
' Wenn ja, so wird die entsprechende "Ereignisprozedur"
' der Objekte aufgerufen
Public Sub OBJ_COLL_OBJ(C1 As ObjType, C2 As ObjType)
  Dim x1 As Long, y1 As Long, x2 As Long, y2 As Long
  Dim h1 As Long, w1 As Long, h2 As Long, w2 As Long
  
    If C1.T = 0 Then Exit Sub
    If C2.T = 0 Then Exit Sub

    x1 = C1.x
    y1 = C1.y
    w1 = C1.FRAME.NACH.w
    h1 = C1.FRAME.NACH.h
    
    x2 = C2.x
    y2 = C2.y
    w2 = C2.FRAME.NACH.w
    h2 = C2.FRAME.NACH.h

    If x1 + w1 - MAX_DIFF > x2 And x1 < x2 + w2 - MAX_DIFF Then
      If y1 + h1 - MAX_DIFF > y2 And y1 < y2 + h2 - MAX_DIFF Then
        Call OBJ_EVENT(C1, ID_COLL, C2, 0, 0)
        Call OBJ_EVENT(C2, ID_COLL, C1, 0, 0)
      End If
    End If
End Sub

Public Function OBJ_DOWN(C As ObjType, D As Long)
  OBJ_DOWN = OBJ_FIELD_ISFREE(C.x, C.y + D, C.FRAME.NACH.w, C.FRAME.NACH.h)
End Function

' Zeichnet das bergebene Objekt relativ zu der Position X,Y
Public Sub OBJ_DRAW(O As ObjType, x As Long, y As Long, PicBox As PictureBox)
  Dim xx As Long, yy As Long
  
    If O.T = 0 Then Exit Sub

    Call OBJ_EVENT(O, ID_DRAW, ObjDummy, 0, 0)

    xx = x + O.x
    yy = y + O.y
    Call SPRITE_DRAW(O.FRAME, xx, yy, PicBox)
End Sub

' Event-Handler fr Objekte
Public Sub OBJ_EVENT(Self As ObjType, MESSAGE, Other As ObjType, _
                     ByVal Par1 As Long, ByVal Par2 As Long)
                     
  Call USER_EVENT(Self, MESSAGE, Other, Par1, Par2)
End Sub

Public Function OBJ_FIELD_ISFREE(x As Long, y As Long, w As Long, h As Long)
  Dim x1 As Long, y1 As Long, x2 As Long, y2 As Long
  Dim aX As Long, aY As Long, fX As Long, fY As Long
  
    x1 = x \ FIELD_W
    y1 = y \ FIELD_H
    x2 = (x + w) \ FIELD_W
    y2 = (y + h) \ FIELD_H
    
    For aX = x1 To x2
      For aY = y1 To y2

        If aX >= 0 And aY >= 0 And aX <= FIELD_X And aY <= FIELD_Y Then
          If FIELD(aX, aY).B And FIELD(aX, aY).V Then
            fX = aX * FIELD_W
            fY = aY * FIELD_H
  
            If x + w - MAX_DIFF > fX And x < fX + FIELD_W - MAX_DIFF Then
              If y + h - MAX_DIFF > fY And y < fY + FIELD_H - MAX_DIFF Then
                OBJ_FIELD_ISFREE = 0
                Exit Function
              End If
            End If
          End If
        Else
          OBJ_FIELD_ISFREE = 0
          Exit Function
        End If
            
      Next aY
    Next aX
                            
    OBJ_FIELD_ISFREE = 1
End Function

' Nimmt dem Objekt das Item weg (falls berhaupt vorhanden)
Public Sub OBJ_ITEM_DROP(O As ObjType, ITEM As Long)
  If O.T = 0 Then Exit Sub
    
  For i = 0 To UBound(O.ITEM)
    If O.ITEM(i).T = ITEM Then
      O.ITEM(i).T = 0
      Exit Sub
    End If
  Next i
End Sub

' Holt die Energie eines Items
Public Function OBJ_ITEM_ENERGY_GET(O As ObjType, ITEM As Long)
  OBJ_ITEM_ENERGY_GET = OBJ_ITEM_HAVE(O, ITEM)
End Function

' ndert die Energie eines Items
Public Sub OBJ_ITEM_ENERGY_SET(O As ObjType, ITEM As Long, Energy As Long)
  If O.T = 0 Then Exit Sub

  For i = 0 To UBound(O.ITEM)
    If O.ITEM(i).T = ITEM Then
      O.ITEM(i).E = Energy
      If Energy <= 0 Then O.ITEM(i).T = 0
      Exit Sub
    End If
  Next i
  Call OBJ_ITEM_GET(O, ITEM, Energy)
End Sub

' Gibt einem Objekt das Item mit der Nummer I
Public Sub OBJ_ITEM_GET(O As ObjType, ITEM As Long, Energy As Long)
  If O.T = 0 Then Exit Sub

  For i = 0 To UBound(O.ITEM)
    If O.ITEM(i).T = 0 Then
      O.ITEM(i).T = ITEM
      O.ITEM(i).E = Energy
      Exit Sub
    End If
  Next i
End Sub

' Liefert zurck, ob das Objekt das entsprechende Item hat
Public Function OBJ_ITEM_HAVE(O As ObjType, ITEM As Long)
  If O.T = 0 Then Exit Function
  
  For i = 0 To UBound(O.ITEM)
    If O.ITEM(i).T = ITEM Then
      OBJ_ITEM_HAVE = O.ITEM(i).E
      Exit Function
    End If
  Next i
End Function

Public Function OBJ_LEFT(C As ObjType, D)
  OBJ_LEFT = OBJ_FIELD_ISFREE(C.x - D, C.y, _
                              C.FRAME.NACH.w, _
                              C.FRAME.NACH.h)
End Function

Public Function OBJ_RIGHT(C As ObjType, D)
  OBJ_RIGHT = OBJ_FIELD_ISFREE(C.x + D, C.y, _
                               C.FRAME.NACH.w, _
                               C.FRAME.NACH.h)
End Function

' Setzt das bergebene Objekt O auf die bergebenen Werte
Public Sub OBJ_SET(O As ObjType, x As Long, y As Long, _
                   E As Long, SX As Long, SY As Long, _
                   T As Long, FRAME As Sprite, V As Long)
  O.x = x
  O.y = y
  O.OX = x
  O.OY = y
  O.E = E
  O.T = T
  O.FRAME = FRAME
  O.V = V
  
  O.SX = SX
  O.SY = SY
End Sub

Public Function OBJ_UP(C As ObjType, D As Long)
  OBJ_UP = OBJ_FIELD_ISFREE(C.x, C.y - D, C.FRAME.NACH.w, C.FRAME.NACH.h)
End Function

